#ifndef __THE_MAZE_OF_GALIOUS_2_OBJECTS
#define __THE_MAZE_OF_GALIOUS_2_OBJECTS

/* MAXIMUM NUMBER OF BITMAPS IN WHICH AN OBJECT CAN BE SPLITTET IN: */ 

#define MAX_PARTS	16
#define MAX_LINKS	16

/* MOG OBJECT CLASS: */ 

#define CONS_NONE	1
#define CONS_SOLID	2
#define CONS_WEAPON	4
#define CONS_LADDER	8
#define CONS_CHARACTER	16
#define CONS_ITEM	32
#define CONS_SOLID_SURFACE	64
#define CONS_ALL	127

#define CTRL_STANDARD	0
#define CTRL_CHARACTER	1
#define CTRL_TEXT		2
#define CTRL_GAMEWINDOW	3
#define CTRL_MENU		4


#define MOVE_NONE		0
#define MOVE_PUSHABLE	1
#define MOVE_SCRIPT		2

#define CPOS_STANDRIGHT			0
#define CPOS_STANDLEFT			1
#define CPOS_PAUSE				2
#define CPOS_WALKINGRIGHT		3
#define CPOS_WALKINGLEFT		4
#define CPOS_FALLINGRIGHT		5
#define CPOS_FALLINGLEFT		6
#define CPOS_LANDINGRIGHT		7
#define CPOS_LANDINGLEFT		8
#define CPOS_RECOVERRIGHT		9
#define CPOS_RECOVERLEFT		10
#define CPOS_DEADRIGHT			11
#define CPOS_DEADLEFT			12

#define CPOS_UPSTAIRS			13
#define CPOS_DOWNSTAIRS			14
#define CPOS_QUIETUPSTAIRS		15
#define CPOS_QUIETDOWNSTAIRS	16
#define CPOS_QUIETRIGHTSTAIRS	17
#define CPOS_QUIETLEFTSTAIRS	18

#define CPOS_JUMPRIGHT			19
#define CPOS_JUMPLEFT			20
#define CPOS_JUMPFALLRIGHT		21
#define CPOS_JUMPFALLLEFT		22

#define CPOS_NSTATES			23


class OBJECT {
	// The EDITOR classes can access all the MOG2 members
	friend class EDITOR;
	friend class RObjectWnd;
	friend class REditWnd;
	friend class RToolsWnd;
	friend class OBitmapWnd;
	friend class OObjectWnd;
	friend class OObjectBmpWnd;
	friend class OPropertiesWnd;
	friend class BAnimEditorWnd;
	friend class BTileViewWnd;
	friend class BTileSelectionWnd;
	friend class HotSpotWnd;
	friend class LinksWnd;
	friend class BGamePreviewDlg;
	friend class OVariableWnd;
	friend class OPartsWnd;
	friend class BOVariableEditorWnd;
	friend class VariableListWnd;
	friend class POVariableWnd;
    friend class BDrawingModeWnd;
	friend class OSampleWnd;
	friend class IObjectWnd;
	friend class IToolsWnd;
	friend class IEditWnd;
	friend class BViewActionScriptWnd;
	friend class BViewValueScriptWnd;

	friend class GameDebugWindow;
	friend class BPlayerConfigDlg;
	friend class GameWindow;
	friend class BNetworkConfigDlg;

	friend class NetworkMOG2Protocol;

	friend class ActiveNetworkMOG2;
	friend class ActiveMOG2Application;
	friend class ActiveMOG2;
	friend class ActiveMAP;
	friend class ActiveROOM;
	friend class ActiveOBJECT;
	friend class ActiveINTERFACE;

	friend class PLACED_OBJECT;

	friend class ROOM;
	friend class MOG2;
	friend class MOG2_INTERFACE;

	friend class ActionSCRIPT;
	friend class ConditionSCRIPT;
	friend class ValueSCRIPT;

	friend bool object_order(OBJECT *o1,OBJECT *o2);

public:
	OBJECT(void);
	OBJECT(char *name);
	OBJECT(char *name,OBJECT *o);
	~OBJECT(void);

	bool save(FILE *fp,class MOG2 *game);
	bool load(FILE *fp,MOG2 *game);

	bool cmp(char *name);
	bool cmp(OBJECT *o);

	void text_initialization(void);
	void text_initialization(OBJECT *o);

	void draw(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp);
	void draw_shaded(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int factor,int r,int g,int b,int a);
	void draw_bicolor(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int factor,int r1,int g1,int b1,int a1,int r2,int g2,int b2,int a2);
	void draw_reflection(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int reflection_kh,int reflection_kv,int reflection_oh,int reflection_ov,int reflection_wave_height,int center_x,int center_y,int factor);
	void draw_refraction(int x,int y,int *tile,int *p_offset_p,SDL_Surface *sfc,SDL_Rect *vp,int refraction_distance,int refraction_kh,int refraction_kv,int refraction_oh,int refraction_ov,int refraction_wave_height,int factor);
	void text_draw(int x,int y,SDL_Surface *sfc,SDL_Rect *vp,MOG2 *game);
	void gw_draw(int x,int y,SDL_Surface *sfc,SDL_Rect *vp);
	void menu_draw(int x,int y,SDL_Surface *sfc,SDL_Rect *vp);

	void draw_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle);
	void draw_shaded_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int factor,int r,int g,int b,int a);
	void draw_bicolor_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int factor,int r1,int g1,int b1,int a1,int r2,int g2,int b2,int a2);
	void draw_reflection_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int reflection_kh,int reflection_kv,int reflection_oh,int reflection_ov,int reflection_wave_height,int center_x,int center_y,int factor);
	void draw_refraction_transform(int x,int y,int *tile,int *p_offset_p,SDL_Surface *dest,SDL_Rect *vp,float scale,float angle,int refraction_distance,int refraction_kh,int refraction_kv,int refraction_oh,int refraction_ov,int refraction_wave_height,int factor);

	void draw_collision_mask(int x,int y,int *tile,SDL_Surface *sfc,SDL_Rect *vp);
	void draw_collision_mask_transform(int x,int y,int *tile,SDL_Surface *sfc,SDL_Rect *vp,float scale,float angle);

	bool passive(void);
	void get_link(int *tile,int part,int n,int *lx,int *ly);

	int get_dx(int *tile);
	int get_dy(int *tile);
	void get_bbox(int x,int y,int *tile,int *x1,int *y1,int *x2,int *y2);
	void get_text_bbox(int x,int y,int *x1,int *y1,int *x2,int *y2,MOG2 *game);
	void get_gw_bbox(int x,int y,int *x1,int *y1,int *x2,int *y2);
	void get_menu_bbox(int x,int y,int *x1,int *y1,int *x2,int *y2);

	void get_part_position(int part,int *tile,int *x,int *y);
	void get_part_position_transform(int part,int *tile,float scale,float angle,int *x,int *y);

	int get_variable_value(char *vname,int *ival,char **sval);
	void set_variable_value(char *vname,int ivalue);
	void set_variable_value(char *vname,char *svalue);

	void get_parts_order(int *order,int *p_offset_p);

	void get_variable_names(List<char> *l);
	
private:
	char *name;
	int ntiles;
	class TILE **tiles;

	/* Complex objects: */ 
	int nparts;
	int *link_part;
	int *link_a,*link_b;
	int **link_x,**link_y;		/* Links for coplex objects	*/ 
	int *nlinks;
	int default_tile[MAX_PARTS];
	int part_precedence[MAX_PARTS];

	/* Kind of object: */ 
	int constitution;
	int control;

	/* Character info: */ 
	class AnimSCRIPT *state_anim[MAX_PARTS][CPOS_NSTATES];
	SAMPLE *state_sample[CPOS_NSTATES];
	int state_sample_command[CPOS_NSTATES];	/* 0 - none				*/ 
											/* 1 - once				*/ 
											/* 2 - repeat			*/ 
											/* 3 - once-end			*/ 
											/* 4 - once(pan)		*/ 
											/* 5 - repeat(pan)		*/ 
											/* 6 - once-end(pan)	*/ 
	int state_sample_command_parameter[CPOS_NSTATES];
	int state_sample_volume[CPOS_NSTATES];
	bool stair_states,jump_states;
	int walking_speed;
	int jumping_ctnt,gravity_ctnt;
	int jumping_key;
	bool variable_jumping,controllable_jump;
	int ladder_up_speed,ladder_down_speed;
	bool stop_at_each_floor;

	/* General info: */ 
	bool active_flag,pushable_flag,multiroom_flag,collision_flag;
	/* General states: */ 
	int n_states;
	char **g_state_description;
	AnimSCRIPT **g_state_anim[MAX_PARTS];
	class ActionSCRIPT **g_state_action; 
	/* General Special states: */ 
	int n_s_states;
	char **g_s_state_description;
	AnimSCRIPT **g_s_state_anim[MAX_PARTS];
	class ActionSCRIPT **g_s_state_action; 
	class ConditionSCRIPT **g_s_state_condition;
	bool *g_s_state_exclusiveness;
	int *g_s_state_priority;

	/* Variables: */ 
	int init_state,init_energy,experience,damage;
	char *damage_type;
	List<VARIABLE> object_variables;

	/* Text variables: */ 
	char *text_text;
	char *text_font;
	int text_size;
	int text_spacing;	/* This is only valid for BITMAP fonts, and specifies the space between characters */ 
	int text_r,text_g,text_b,text_a;
	int text_frame;	/* 0 - No frame		*/ 
					/* 1 - Square frame */ 
					/* 2 - Balloon		*/ 
	int text_frame_width;
	int text_frame_r1,text_frame_g1,text_frame_b1,text_frame_a1;
	int text_frame_r2,text_frame_g2,text_frame_b2,text_frame_a2;
	int text_frame_radius;
	int text_frame_point_x,text_frame_point_y;
	int text_h_align;	/* 0 - Left
						   1 - Center
						   2 - Right */ 
	int text_v_align;	/* 0 - Top
						   1 - Center
						   2 - Bottom */ 
	int text_max_w;

	/* game window variables: */ 
	int gw_dx,gw_dy;
	int gw_scale;
	int gw_min_hs_x,gw_min_hs_y;
	int gw_max_hs_x,gw_max_hs_y;

	/* menu variables: */ 
	int menu_dx;	/* this variable is just orientative! */ 
	int menu_dy;	/* this variable is just orientative! */ 
	char *menu_title;
	int menu_appearance_flags;
	int menu_entry_effect_flags;
	int menu_blink_timmer,menu_zoom_factor,menu_appear_factor;	/* effect parameters */ 
	int menu_text_r2,menu_text_g2,menu_text_b2,menu_text_a2;

        
	/* Default Drawing MODE: */ 
	int dm_drawing_mode;	/* 0 - normal   */ 
							/* 1 - shaded   */ 
						    /* 2 - bicolor	*/ 
							/* 3 - reflection	*/ 
							/* 4 - refraction	*/ 
	int dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a;
	int dm_color2_r,dm_color2_g,dm_color2_b,dm_color2_a;
	int dm_factor;
	int dm_blink_f0,dm_blink_f1,dm_blink_speed;

	/* Default scaling factor and rotation: */ 
	int scale,angle;
};


class PLACED_OBJECT {
	// The EDITOR classes can access all the MOG2 members
	friend class EDITOR;
	friend class REditWnd;
	friend class RToolsWnd;
	friend class POVariableWnd;
	friend class BRMoveDlg;
	friend class IToolsWnd;
	friend class IEditWnd;

	friend class ActiveMOG2;
	friend class ActiveMAP;
	friend class ActiveROOM;
	friend class ActiveOBJECT;
	friend class ActiveINTERFACE;

	friend class ROOM;

public:
	PLACED_OBJECT(void);
	PLACED_OBJECT(long int x,long int y,OBJECT *o,MOG2 *g,ROOM *r,MOG2_INTERFACE *i);
	PLACED_OBJECT(PLACED_OBJECT &o);
	~PLACED_OBJECT(void);

	bool save(FILE *fp,MOG2 *game);
	bool load(FILE *fp,MOG2 *game,ROOM *r,MOG2_INTERFACE *i);

	/* These functions are only used by the Editor, never by the engine itself: */  
	void draw(SDL_Surface *sfc);
	void draw(long int sx,long int sy,SDL_Surface *sfc);
	void draw_scaled(long int sx,long int sy,SDL_Surface *dest,float scale);
	void draw_collision_mask(long int sx,long int sy,SDL_Surface *sfc);

	void text_draw(long int sx,long int sy,SDL_Surface *sfc,SDL_Rect *vp,MOG2 *game);

	int get_dx(void);
	int get_dy(void);

	int get_x(void) {return x;};
	int get_y(void) {return y;};

	void get_text_bbox(int *x1,int *y1,int *x2,int *y2);
	void get_gw_bbox(int *x1,int *y1,int *x2,int *y2);
	void get_menu_bbox(int *x1,int *y1,int *x2,int *y2);

private:
	int ID;
	long int x,y;
	OBJECT *object;
	MOG2 *game;
	ROOM *room;			/* room = 0 XOR interf = 0 */ 
	MOG2_INTERFACE *interf;

	List<VARIABLE> modified_variables;	/* This list contains the list of variables	*/ 
										/* that have a different initial value than	*/ 
										/* the default one.							*/ 
};


#endif

